capture program  drop distance_ord
program distance_ord , rclass
* Version 1.0.0
* 28jan2020
* Author: Lucas van der Velde (lvandervelde@grape.org.pl)

syntax varlist(min=1) [if], [AVErage(varname) Weights(numlist) Pattern(numlist) NOPairwise] 


** Selecting the observations for the analysis
*--------------------------------------------------
 
quiet{
tempvar samples
gen `samples' = 1  `if' 
 gsort - `samples'

 

preserve
keep if `samples'==1
keep `varlist' 
}
 
local nobs  =_N
local nvars : word count `varlist'


tempname wx1 pp1 output prods num1 C D output2

*	Errors
*-----------------------------------

if "`nopairwise'" !="" & "`average'"!= ""{
		di as error "Option average requires that all pairwise comparisons are run"
		exit
}

if "`weights'"!=""{
local nweights : word count `weights'

	if `nweights'!=`nvars' {
		di as error "The number of variables and the number of weights should match"
		exit
		}


	foreach el in `weights'{
		local sums = `sums'+ `el'
		matrix `wx1' = nullmat(`wx1'),`el'
		}
	
		matrix `wx1' = `wx1'/`sums'*`nweights'
	}
if "`weights'"==""{
	matrix `wx1' = J(1,`nvars',1)
}	


if "`pattern'"!=""{
	local npatt : word count `pattern'
	if `npatt'!=`nvars' {
		di as error "The number of target values and the number of variables should match"
		exit
		}

	foreach el in `pattern'{
		matrix `pp1' = nullmat(`pp1') \ `el'
		}
		
		
	}

** Obtaining measures of distance across observations
*------------------------------------------------------
 
 xpose, clear
 
 if "`pattern'"!=""{
	local oo = 2
	local nobs2 = `nobs'+1
	svmat `pp1' , name(los) 
	ren los v`nobs2'
	}
 
 
 forvalues k = 1/`nobs`oo''{

 capture drop dist*
  
 forvalues l  = 1/`nobs`oo''{
 	gen dist`l' = sign(v`k' - v`l')
 }

tempname m`k' 
mkmat dist1-dist`nobs`oo'', mat(`m`k'')
matrix `m`k'' = `m`k'''
 }
 


** Computing final distances
*-------------------------------------

matrix `C' = J(`nvars',1,1)
matrix `D' = J(1,`nobs',1)

matrix `output'	= J(`nobs',`nobs',.)
matrix `prods' 	= J(`nobs',`nobs',.)

forvalues i = 1/`nobs'{
	forvalues j = 1/`nobs'{
		matrix `prods'[`i',`j'] = `D'*hadamard(`m`i''[1..`nobs',1...],`m`j''[1..`nobs',1...])*`wx1'' 
	}
}

** All pairwise comparisons	
if "`nopairwise'" == "" {
forvalues i = 1/`nobs'{
	forvalues j = `i'/`nobs'{
		mat `num1' = hadamard(`m`i''[`j',1...],`m`i''[`j',1...])*`wx1''
		
		mat `output'[`i',`j'] = 1/2*(1 - (-`num1'[1,1] + `prods'[`i',`j'])/sqrt(`prods'[`i',`i']*`prods'[`j',`j']))
		mat `output'[`j',`i'] = `output'[`i',`j']
	}
 }
local ll  "lucas"
 }

 ** Compare everything to reference value
 if "`pattern'" != "" {
 
	tempname D2 output2 prodkk prodik prodii
	
	matrix `D2' = J(1,`nobs2',1)
	matrix `output2' = J(1,`nobs',.)
    matrix `prodkk' = `D2'*hadamard(`m`nobs2'',`m`nobs2'')*`wx1'' 
 
 forvalues i = 1/`nobs'{
		matrix `num1' = hadamard(`m`nobs2''[`i',1...],`m`nobs2''[`i',1...])*`wx1''
		matrix `prodik' = `D2'*hadamard(`m`i'',`m`nobs2'')*`wx1''
		matrix `prodii' = `D2'*hadamard(`m`i'',`m`i'')*`wx1''
		
		matrix `output2'[1,`i'] = 1/2*(1 - (-`num1'[1,1] + `prodik'[1,1])/sqrt(`prodii'[1,1]*`prodkk'[1,1]))
		}
 
 
 }
 
 
 restore
 
    
 *	Calculating average distance (optional)
 *------------------------------------------
 
if "`average'"!=""{
 
 forvalues i = 1/`nobs'{
	matrix x = `output'[`i',1...]*`D''
	quiet replace `average' = x[1,1]/`nobs' if _n==`i'
 }
 
}

if "`nopairwise'"=="" 	return matrix  pairwise = `output'
if "`pattern'"!="" return matrix  direct = `output2'

mat l  `wx1'
end

